
import 'distance_between.dart';

class Vector2{
  double x;
  double y;
  Vector2(this.x, this.y);
  bool get isZero => x == 0 && y == 0;
}

T findClosest<T extends Vector2>(List<T> values, double x, double y){
  if (values.isEmpty){
    throw Exception("findClosest.error.values is empty");
  }
  T closest = values[0];
  double closestDistance = distanceBetween(closest.x, closest.y, x, y);

  for(int i = 1; i < values.length; i++){
    T t = values[i];
    double tDistance = distanceBetween(t.x, t.y, x, y);
    if (tDistance < closestDistance){
      closest = t;
      closestDistance = tDistance;
    }
  }
  return closest;
}

/// Insertion sort tailored for vectors
void sortVertically(List<Vector2> items) {
  int start = 0;
  int end = items.length;
  for (var pos = start + 1; pos < end; pos++) {
    var min = start;
    var max = pos;
    var element = items[pos];
    while (min < max) {
      var mid = min + ((max - min) >> 1);
      if (element.y < items[mid].y) {
        max = mid;
      } else {
        min = mid + 1;
      }
    }
    items.setRange(min + 1, pos + 1, items, min);
    items[min] = element;
  }
}

Vector2? getClosest(double x, double y, List<List<Vector2>> lists){
  final List<Vector2> close = [];
  for(final list in lists){
    if (list.isEmpty) continue;
    close.add(findClosest(list, x, y));
  }
  if (close.isEmpty) return null;
  return findClosest(close, x, y);
}



